@@ -15,37 +15,45 @@
 
 """Identity v2 Token action implementations"""
 
-import logging
+from osc_lib.command import command
+from osc_lib import exceptions
 import six
 
-from cliff import command
-from cliff import show
-
-from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
+from openstackclient.i18n import _
 
 
-class IssueToken(show.ShowOne):
-    """Issue new token"""
+class IssueToken(command.ShowOne):
+    _description = _("Issue new token")
 
-    log = logging.getLogger(__name__ + '.IssueToken')
+    # scoped token is optional
+    required_scope = False
 
     def get_parser(self, prog_name):
         parser = super(IssueToken, self).get_parser(prog_name)
         return parser
 
-    @utils.log_method(log)
     def take_action(self, parsed_args):
+        auth_ref = self.app.client_manager.auth_ref
+        if not auth_ref:
+            raise exceptions.AuthorizationFailure(
+                "Only an authorized user may issue a new token.")
 
-        token = self.app.client_manager.auth_ref.service_catalog.get_token()
-        token['project_id'] = token.pop('tenant_id')
-        return zip(*sorted(six.iteritems(token)))
+        data = {}
+        if auth_ref.auth_token:
+            data['id'] = auth_ref.auth_token
+        if auth_ref.expires:
+            datetime_obj = auth_ref.expires
+            expires_str = datetime_obj.strftime('%Y-%m-%dT%H:%M:%S%z')
+            data['expires'] = expires_str
+        if auth_ref.project_id:
+            data['project_id'] = auth_ref.project_id
+        if auth_ref.user_id:
+            data['user_id'] = auth_ref.user_id
+        return zip(*sorted(six.iteritems(data)))
 
 
 class RevokeToken(command.Command):
-    """Revoke existing token"""
-
-    log = logging.getLogger(__name__ + '.RevokeToken')
+    _description = _("Revoke existing token")
 
     def get_parser(self, prog_name):
         parser = super(RevokeToken, self).get_parser(prog_name)
@@ -57,8 +65,6 @@ class RevokeToken(command.Command):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
         identity_client = self.app.client_manager.identity
 
         identity_client.tokens.delete(parsed_args.token)
-        return
