@@ -16,25 +16,32 @@
 """Image V2 Action Implementations"""
 
 import argparse
-import six
+import logging
 
 from glanceclient.common import utils as gc_utils
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+import six
 
 from openstackclient.api import utils as api_utils
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import parseractions
-from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
+from openstackclient.i18n import _
 from openstackclient.identity import common
 
 
+CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
+DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
+                "vdi", "iso", "ploop"]
+
+
+LOG = logging.getLogger(__name__)
 
 
 def _format_image(image):
-    """Format an image to make it more consistent with OSC operations. """
+    """Format an image to make it more consistent with OSC operations."""
 
     info = {}
     properties = {}
@@ -65,19 +72,19 @@ def _format_image(image):
 
 
 class AddProjectToImage(command.ShowOne):
-    """Associate project with image"""
+    _description = _("Associate project with image")
 
     def get_parser(self, prog_name):
         parser = super(AddProjectToImage, self).get_parser(prog_name)
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to share (name or ID)",
+            help=_("Image to share (name or ID)"),
         )
         parser.add_argument(
             "project",
             metavar="<project>",
-            help="Project to associate with image (name or ID)",
+            help=_("Project to associate with image (name or ID)"),
         )
         common.add_project_domain_option_to_parser(parser)
         return parser
@@ -103,7 +110,7 @@ class AddProjectToImage(command.ShowOne):
 
 
 class CreateImage(command.ShowOne):
-    """Create/upload an image"""
+    _description = _("Create/upload an image")
 
     deadopts = ('size', 'location', 'copy-from', 'checksum', 'store')
 
@@ -119,94 +126,110 @@ class CreateImage(command.ShowOne):
         parser.add_argument(
             "name",
             metavar="<image-name>",
-            help="New image name",
+            help=_("New image name"),
         )
         parser.add_argument(
             "--id",
             metavar="<id>",
-            help="Image ID to reserve",
+            help=_("Image ID to reserve"),
         )
         parser.add_argument(
             "--container-format",
             default=DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
             metavar="<container-format>",
-            help="Image container format "
-                 "(default: %s)" % DEFAULT_CONTAINER_FORMAT,
+            help=(_("Image container format. "
+                    "The supported options are: %(option_list)s. "
+                    "The default format is: %(default_opt)s") %
+                  {'option_list': ', '.join(CONTAINER_CHOICES),
+                   'default_opt': DEFAULT_CONTAINER_FORMAT})
         )
         parser.add_argument(
             "--disk-format",
             default=DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
             metavar="<disk-format>",
-            help="Image disk format "
-                 "(default: %s)" % DEFAULT_DISK_FORMAT,
+            help=_("Image disk format. The supported options are: %s. "
+                   "The default format is: raw") % ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--min-disk",
             metavar="<disk-gb>",
             type=int,
-            help="Minimum disk size needed to boot image, in gigabytes",
+            help=_("Minimum disk size needed to boot image, in gigabytes"),
         )
         parser.add_argument(
             "--min-ram",
             metavar="<ram-mb>",
             type=int,
-            help="Minimum RAM size needed to boot image, in megabytes",
+            help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             "--file",
             metavar="<file>",
-            help="Upload image from local file",
+            help=_("Upload image from local file"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--volume",
             metavar="<volume>",
-            help="Create image from a volume",
+            help=_("Create image from a volume"),
         )
         parser.add_argument(
             "--force",
             dest='force',
             action='store_true',
             default=False,
-            help="Force image creation if volume is in use "
-            "(only meaningful with --volume)",
+            help=_("Force image creation if volume is in use "
+                   "(only meaningful with --volume)"),
         )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
             action="store_true",
-            help="Prevent image from being deleted",
+            help=_("Prevent image from being deleted"),
         )
         protected_group.add_argument(
             "--unprotected",
             action="store_true",
-            help="Allow image to be deleted (default)",
+            help=_("Allow image to be deleted (default)"),
         )
         public_group = parser.add_mutually_exclusive_group()
         public_group.add_argument(
             "--public",
             action="store_true",
-            help="Image is accessible to the public",
+            help=_("Image is accessible to the public"),
         )
         public_group.add_argument(
             "--private",
             action="store_true",
-            help="Image is inaccessible to the public (default)",
+            help=_("Image is inaccessible to the public (default)"),
+        )
+        public_group.add_argument(
+            "--community",
+            action="store_true",
+            help=_("Image is accessible to the community"),
+        )
+        public_group.add_argument(
+            "--shared",
+            action="store_true",
+            help=_("Image can be shared"),
         )
         parser.add_argument(
             "--property",
             dest="properties",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help="Set a property on this image "
-                 "(repeat option to set multiple properties)",
+            help=_("Set a property on this image "
+                   "(repeat option to set multiple properties)"),
         )
         parser.add_argument(
             "--tag",
             dest="tags",
             metavar="<tag>",
             action='append',
-            help="Set a tag on this image "
-                 "(repeat option to set multiple tags)",
+            help=_("Set a tag on this image "
+                   "(repeat option to set multiple tags)"),
         )
         # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
         #                2.x release.  Do not remove before Jan 2017
@@ -215,7 +238,7 @@ class CreateImage(command.ShowOne):
         project_group.add_argument(
             "--project",
             metavar="<project>",
-            help="Set an alternate project on this image (name or ID)",
+            help=_("Set an alternate project on this image (name or ID)"),
         )
         project_group.add_argument(
             "--owner",
@@ -239,15 +262,15 @@ class CreateImage(command.ShowOne):
         for deadopt in self.deadopts:
             if getattr(parsed_args, deadopt.replace('-', '_'), None):
                 raise exceptions.CommandError(
-                    "ERROR: --%s was given, which is an Image v1 option"
-                    " that is no longer supported in Image v2" % deadopt)
+                    _("ERROR: --%s was given, which is an Image v1 option"
+                      " that is no longer supported in Image v2") % deadopt)
 
         # Build an attribute dict from the parsed args, only include
         # attributes that were actually set on the command line
         kwargs = {}
         copy_attrs = ('name', 'id',
                       'container_format', 'disk_format',
-                      'min_disk', 'min_ram', 'tags')
+                      'min_disk', 'min_ram', 'tags', 'visibility')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
@@ -275,15 +298,16 @@ class CreateImage(command.ShowOne):
             kwargs['visibility'] = 'public'
         if parsed_args.private:
             kwargs['visibility'] = 'private'
-
+        if parsed_args.community:
+            kwargs['visibility'] = 'community'
+        if parsed_args.shared:
+            kwargs['visibility'] = 'shared'
         # Handle deprecated --owner option
         project_arg = parsed_args.project
         if parsed_args.owner:
             project_arg = parsed_args.owner
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
+            LOG.warning(_('The --owner option is deprecated, '
+                          'please use --project instead.'))
         if project_arg:
             kwargs['owner'] = common.find_project(
                 identity_client,
@@ -296,11 +320,12 @@ class CreateImage(command.ShowOne):
         fp = gc_utils.get_data_file(parsed_args)
         info = {}
         if fp is not None and parsed_args.volume:
-            raise exceptions.CommandError("Uploading data and using container "
-                                          "are not allowed at the same time")
+            raise exceptions.CommandError(_("Uploading data and using "
+                                            "container are not allowed at "
+                                            "the same time"))
 
         if fp is None and parsed_args.file:
-            self.log.warning("Failed to get an image file.")
+            LOG.warning(_("Failed to get an image file."))
             return {}, {}
 
         if parsed_args.owner:
@@ -325,7 +350,10 @@ class CreateImage(command.ShowOne):
                 parsed_args.disk_format,
             )
             info = body['os-volume_upload_image']
-            info['volume_type'] = info['volume_type']['name']
+            try:
+                info['volume_type'] = info['volume_type']['name']
+            except TypeError:
+                info['volume_type'] = None
         else:
             image = image_client.images.create(**kwargs)
 
@@ -333,7 +361,7 @@ class CreateImage(command.ShowOne):
             with fp:
                 try:
                     image_client.images.upload(image.id, fp)
-                except Exception as e:
+                except Exception:
                     # If the upload fails for some reason attempt to remove the
                     # dangling queued image made by the create() call above but
                     # only if the user did not specify an id which indicates
@@ -343,7 +371,7 @@ class CreateImage(command.ShowOne):
                             image_client.images.delete(image.id)
                     except Exception:
                         pass  # we don't care about this one
-                    raise e  # now, throw the upload exception again
+                    raise  # now, throw the upload exception again
 
                 # update the image after the data has been uploaded
                 image = image_client.images.get(image.id)
@@ -355,7 +383,7 @@ class CreateImage(command.ShowOne):
 
 
 class DeleteImage(command.Command):
-    """Delete image(s)"""
+    _description = _("Delete image(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteImage, self).get_parser(prog_name)
@@ -363,22 +391,36 @@ class DeleteImage(command.Command):
             "images",
             metavar="<image>",
             nargs="+",
-            help="Image(s) to delete (name or ID)",
+            help=_("Image(s) to delete (name or ID)"),
         )
         return parser
 
     def take_action(self, parsed_args):
+
+        del_result = 0
         image_client = self.app.client_manager.image
         for image in parsed_args.images:
-            image_obj = utils.find_resource(
-                image_client.images,
-                image,
-            )
-            image_client.images.delete(image_obj.id)
+            try:
+                image_obj = utils.find_resource(
+                    image_client.images,
+                    image,
+                )
+                image_client.images.delete(image_obj.id)
+            except Exception as e:
+                del_result += 1
+                LOG.error(_("Failed to delete image with name or "
+                            "ID '%(image)s': %(e)s"),
+                          {'image': image, 'e': e})
+
+        total = len(parsed_args.images)
+        if (del_result > 0):
+            msg = (_("Failed to delete %(dresult)s of %(total)s images.")
+                   % {'dresult': del_result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListImage(command.Lister):
-    """List available images"""
+    _description = _("List available images")
 
     def get_parser(self, prog_name):
         parser = super(ListImage, self).get_parser(prog_name)
@@ -388,33 +430,58 @@ class ListImage(command.Lister):
             dest="public",
             action="store_true",
             default=False,
-            help="List only public images",
+            help=_("List only public images"),
         )
         public_group.add_argument(
             "--private",
             dest="private",
             action="store_true",
             default=False,
-            help="List only private images",
+            help=_("List only private images"),
+        )
+        public_group.add_argument(
+            "--community",
+            dest="community",
+            action="store_true",
+            default=False,
+            help=_("List only community images"),
         )
         public_group.add_argument(
             "--shared",
             dest="shared",
             action="store_true",
             default=False,
-            help="List only shared images",
+            help=_("List only shared images"),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Filter output based on property',
+            help=_('Filter output based on property'),
+        )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            default=None,
+            help=_("Filter images based on name.")
+        )
+        parser.add_argument(
+            '--status',
+            metavar='<status>',
+            default=None,
+            help=_("Filter images based on status.")
+        )
+        parser.add_argument(
+            '--tag',
+            metavar='<tag>',
+            default=None,
+            help=_('Filter images based on tag.'),
         )
         parser.add_argument(
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output',
+            help=_('List additional fields in output'),
         )
 
         # --page-size has never worked, leave here for silent compatibility
@@ -427,23 +494,24 @@ class ListImage(command.Lister):
         parser.add_argument(
             '--sort',
             metavar="<key>[:<direction>]",
-            help="Sort output by selected keys and directions(asc or desc) "
-                 "(default: asc), multiple keys and directions can be "
-                 "specified separated by comma",
+            default='name:asc',
+            help=_("Sort output by selected keys and directions(asc or desc) "
+                   "(default: name:asc), multiple keys and directions can be "
+                   "specified separated by comma"),
         )
         parser.add_argument(
             "--limit",
-            metavar="<limit>",
+            metavar="<num-images>",
             type=int,
-            help="Maximum number of images to display.",
+            help=_("Maximum number of images to display."),
         )
         parser.add_argument(
             '--marker',
-            metavar='<marker>',
+            metavar='<image>',
             default=None,
-            help="The last image (name or ID) of the previous page. Display "
-                 "list of images after marker. Display all images if not "
-                 "specified."
+            help=_("The last image of the previous page. Display "
+                   "list of images after marker. Display all images if not "
+                   "specified. (name or ID)"),
         )
         return parser
 
@@ -455,6 +523,8 @@ class ListImage(command.Lister):
             kwargs['public'] = True
         if parsed_args.private:
             kwargs['private'] = True
+        if parsed_args.community:
+            kwargs['community'] = True
         if parsed_args.shared:
             kwargs['shared'] = True
         if parsed_args.limit:
@@ -462,7 +532,12 @@ class ListImage(command.Lister):
         if parsed_args.marker:
             kwargs['marker'] = utils.find_resource(image_client.images,
                                                    parsed_args.marker).id
-
+        if parsed_args.name:
+            kwargs['name'] = parsed_args.name
+        if parsed_args.status:
+            kwargs['status'] = parsed_args.status
+        if parsed_args.tag:
+            kwargs['tag'] = parsed_args.tag
         if parsed_args.long:
             columns = (
                 'ID',
@@ -470,6 +545,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'visibility',
                 'protected',
@@ -482,6 +558,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'Visibility',
                 'Protected',
@@ -493,7 +570,19 @@ class ListImage(command.Lister):
             column_headers = columns
 
         # List of image data received
-        data = image_client.api.image_list(**kwargs)
+        data = []
+        if 'marker' in kwargs:
+            data = image_client.api.image_list(**kwargs)
+        else:
+            # No pages received yet, so start the page marker at None.
+            marker = None
+            while True:
+                page = image_client.api.image_list(marker=marker, **kwargs)
+                if not page:
+                    break
+                data.extend(page)
+                # Set the marker to the id of the last item we received
+                marker = page[-1]['id']
 
         if parsed_args.property:
             # NOTE(dtroyer): coerce to a list to subscript it in py3
@@ -505,7 +594,7 @@ class ListImage(command.Lister):
                 property_field='properties',
             )
 
-        data = utils.sort_items(data, parsed_args.sort)
+        data = utils.sort_items(data, parsed_args.sort, str)
 
         return (
             column_headers,
@@ -519,20 +608,53 @@ class ListImage(command.Lister):
         )
 
 
+class ListImageProjects(command.Lister):
+    _description = _("List projects associated with image")
+
+    def get_parser(self, prog_name):
+        parser = super(ListImageProjects, self).get_parser(prog_name)
+        parser.add_argument(
+            "image",
+            metavar="<image>",
+            help=_("Image (name or ID)"),
+        )
+        common.add_project_domain_option_to_parser(parser)
+        return parser
+
+    def take_action(self, parsed_args):
+        image_client = self.app.client_manager.image
+        columns = (
+            "Image ID",
+            "Member ID",
+            "Status"
+        )
+
+        image_id = utils.find_resource(
+            image_client.images,
+            parsed_args.image).id
+
+        data = image_client.image_members.list(image_id)
+
+        return (columns,
+                (utils.get_item_properties(
+                    s, columns,
+                ) for s in data))
+
+
 class RemoveProjectImage(command.Command):
-    """Disassociate project with image"""
+    _description = _("Disassociate project with image")
 
     def get_parser(self, prog_name):
         parser = super(RemoveProjectImage, self).get_parser(prog_name)
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to unshare (name or ID)",
+            help=_("Image to unshare (name or ID)"),
         )
         parser.add_argument(
             "project",
             metavar="<project>",
-            help="Project to disassociate with image (name or ID)",
+            help=_("Project to disassociate with image (name or ID)"),
         )
         common.add_project_domain_option_to_parser(parser)
         return parser
@@ -553,19 +675,19 @@ class RemoveProjectImage(command.Command):
 
 
 class SaveImage(command.Command):
-    """Save an image locally"""
+    _description = _("Save an image locally")
 
     def get_parser(self, prog_name):
         parser = super(SaveImage, self).get_parser(prog_name)
         parser.add_argument(
             "--file",
             metavar="<filename>",
-            help="Downloaded image save filename (default: stdout)",
+            help=_("Downloaded image save filename (default: stdout)"),
         )
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to save (name or ID)",
+            help=_("Image to save (name or ID)"),
         )
         return parser
 
@@ -577,11 +699,17 @@ class SaveImage(command.Command):
         )
         data = image_client.images.data(image.id)
 
+        if data.wrapped is None:
+            msg = _('Image %s has no data.') % image.id
+            LOG.error(msg)
+            self.app.stdout.write(msg + '\n')
+            raise SystemExit
+
         gc_utils.save_image(data, parsed_args.file)
 
 
 class SetImage(command.Command):
-    """Set image properties"""
+    _description = _("Set image properties")
 
     deadopts = ('visibility',)
 
@@ -600,85 +728,97 @@ class SetImage(command.Command):
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to modify (name or ID)"
+            help=_("Image to modify (name or ID)")
         )
         parser.add_argument(
             "--name",
             metavar="<name>",
-            help="New image name"
+            help=_("New image name")
         )
         parser.add_argument(
             "--min-disk",
             type=int,
             metavar="<disk-gb>",
-            help="Minimum disk size needed to boot image, in gigabytes"
+            help=_("Minimum disk size needed to boot image, in gigabytes")
         )
         parser.add_argument(
             "--min-ram",
             type=int,
             metavar="<ram-mb>",
-            help="Minimum RAM size needed to boot image, in megabytes",
+            help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
         parser.add_argument(
             "--container-format",
             metavar="<container-format>",
-            help="Image container format "
-                 "(default: %s)" % DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
+            help=_("Image container format. The supported options are: %s") %
+            ', '.join(CONTAINER_CHOICES)
         )
         parser.add_argument(
             "--disk-format",
             metavar="<disk-format>",
-            help="Image disk format "
-                 "(default: %s)" % DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s") %
+            ', '.join(DISK_CHOICES)
         )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
             action="store_true",
-            help="Prevent image from being deleted",
+            help=_("Prevent image from being deleted"),
         )
         protected_group.add_argument(
             "--unprotected",
             action="store_true",
-            help="Allow image to be deleted (default)",
+            help=_("Allow image to be deleted (default)"),
         )
         public_group = parser.add_mutually_exclusive_group()
         public_group.add_argument(
             "--public",
             action="store_true",
-            help="Image is accessible to the public",
+            help=_("Image is accessible to the public"),
         )
         public_group.add_argument(
             "--private",
             action="store_true",
-            help="Image is inaccessible to the public (default)",
+            help=_("Image is inaccessible to the public (default)"),
+        )
+        public_group.add_argument(
+            "--community",
+            action="store_true",
+            help=_("Image is accessible to the community"),
+        )
+        public_group.add_argument(
+            "--shared",
+            action="store_true",
+            help=_("Image can be shared"),
         )
         parser.add_argument(
             "--property",
             dest="properties",
             metavar="<key=value>",
             action=parseractions.KeyValueAction,
-            help="Set a property on this image "
-                 "(repeat option to set multiple properties)",
+            help=_("Set a property on this image "
+                   "(repeat option to set multiple properties)"),
         )
         parser.add_argument(
             "--tag",
             dest="tags",
             metavar="<tag>",
-            default=[],
+            default=None,
             action='append',
-            help="Set a tag on this image "
-                 "(repeat option to set multiple tags)",
+            help=_("Set a tag on this image "
+                   "(repeat option to set multiple tags)"),
         )
         parser.add_argument(
             "--architecture",
             metavar="<architecture>",
-            help="Operating system architecture",
+            help=_("Operating system architecture"),
         )
         parser.add_argument(
             "--instance-id",
             metavar="<instance-id>",
-            help="ID of server instance used to create this image",
+            help=_("ID of server instance used to create this image"),
         )
         parser.add_argument(
             "--instance-uuid",
@@ -689,33 +829,33 @@ class SetImage(command.Command):
         parser.add_argument(
             "--kernel-id",
             metavar="<kernel-id>",
-            help="ID of kernel image used to boot this disk image",
+            help=_("ID of kernel image used to boot this disk image"),
         )
         parser.add_argument(
             "--os-distro",
             metavar="<os-distro>",
-            help="Operating system distribution name",
+            help=_("Operating system distribution name"),
         )
         parser.add_argument(
             "--os-version",
             metavar="<os-version>",
-            help="Operating system distribution version",
+            help=_("Operating system distribution version"),
         )
         parser.add_argument(
             "--ramdisk-id",
             metavar="<ramdisk-id>",
-            help="ID of ramdisk image used to boot this disk image",
+            help=_("ID of ramdisk image used to boot this disk image"),
         )
         deactivate_group = parser.add_mutually_exclusive_group()
         deactivate_group.add_argument(
             "--deactivate",
             action="store_true",
-            help="Deactivate the image",
+            help=_("Deactivate the image"),
         )
         deactivate_group.add_argument(
             "--activate",
             action="store_true",
-            help="Activate the image",
+            help=_("Activate the image"),
         )
         # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
         #                2.x release.  Do not remove before Jan 2017
@@ -724,7 +864,7 @@ class SetImage(command.Command):
         project_group.add_argument(
             "--project",
             metavar="<project>",
-            help="Set an alternate project on this image (name or ID)",
+            help=_("Set an alternate project on this image (name or ID)"),
         )
         project_group.add_argument(
             "--owner",
@@ -739,6 +879,23 @@ class SetImage(command.Command):
                 dest=deadopt.replace('-', '_'),
                 help=argparse.SUPPRESS,
             )
+
+        membership_group = parser.add_mutually_exclusive_group()
+        membership_group.add_argument(
+            "--accept",
+            action="store_true",
+            help=_("Accept the image membership"),
+        )
+        membership_group.add_argument(
+            "--reject",
+            action="store_true",
+            help=_("Reject the image membership"),
+        )
+        membership_group.add_argument(
+            "--pending",
+            action="store_true",
+            help=_("Reset the image membership to 'pending'"),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -748,18 +905,18 @@ class SetImage(command.Command):
         for deadopt in self.deadopts:
             if getattr(parsed_args, deadopt.replace('-', '_'), None):
                 raise exceptions.CommandError(
-                    "ERROR: --%s was given, which is an Image v1 option"
-                    " that is no longer supported in Image v2" % deadopt)
+                    _("ERROR: --%s was given, which is an Image v1 option"
+                      " that is no longer supported in Image v2") % deadopt)
 
         kwargs = {}
         copy_attrs = ('architecture', 'container_format', 'disk_format',
                       'file', 'instance_id', 'kernel_id', 'locations',
                       'min_disk', 'min_ram', 'name', 'os_distro', 'os_version',
-                      'prefix', 'progress', 'ramdisk_id', 'tags')
+                      'prefix', 'progress', 'ramdisk_id', 'tags', 'visibility')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
-                if val:
+                if val is not None:
                     # Only include a value in kwargs for attributes that are
                     # actually present on the command line
                     kwargs[attr] = val
@@ -783,26 +940,24 @@ class SetImage(command.Command):
             kwargs['visibility'] = 'public'
         if parsed_args.private:
             kwargs['visibility'] = 'private'
-
+        if parsed_args.community:
+            kwargs['visibility'] = 'community'
+        if parsed_args.shared:
+            kwargs['visibility'] = 'shared'
         # Handle deprecated --owner option
         project_arg = parsed_args.project
         if parsed_args.owner:
             project_arg = parsed_args.owner
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
+            LOG.warning(_('The --owner option is deprecated, '
+                          'please use --project instead.'))
+        project_id = None
         if project_arg:
-            kwargs['owner'] = common.find_project(
+            project_id = common.find_project(
                 identity_client,
                 project_arg,
                 parsed_args.project_domain,
             ).id
-
-        # Checks if anything that requires getting the image
-        if not (kwargs or parsed_args.deactivate or parsed_args.activate):
-            self.log.warning("No arguments specified")
-            return {}, {}
+            kwargs['owner'] = project_id
 
         image = utils.find_resource(
             image_client.images, parsed_args.image)
@@ -815,9 +970,20 @@ class SetImage(command.Command):
             image_client.images.reactivate(image.id)
             activation_status = "activated"
 
-        # Check if need to do the actual update
-        if not kwargs:
-            return {}, {}
+        membership_group_args = ('accept', 'reject', 'pending')
+        membership_status = [status for status in membership_group_args
+                             if getattr(parsed_args, status)]
+        if membership_status:
+            # If a specific project is not passed, assume we want to update
+            # our own membership
+            if not project_id:
+                project_id = self.app.client_manager.auth_ref.project_id
+            # The mutually exclusive group of the arg parser ensure we have at
+            # most one item in the membership_status list.
+            if membership_status[0] != 'pending':
+                membership_status[0] += 'ed'  # Glance expects the past form
+            image_client.image_members.update(
+                image.id, project_id, membership_status[0])
 
         if parsed_args.tags:
             # Tags should be extended, but duplicates removed
@@ -825,21 +991,71 @@ class SetImage(command.Command):
 
         try:
             image = image_client.images.update(image.id, **kwargs)
-        except Exception as e:
+        except Exception:
             if activation_status is not None:
-                print("Image %s was %s." % (image.id, activation_status))
-            raise e
+                LOG.info(_("Image %(id)s was %(status)s."),
+                         {'id': image.id, 'status': activation_status})
+            raise
 
 
 class ShowImage(command.ShowOne):
-    """Display image details"""
+    _description = _("Display image details")
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "--human-readable",
+            default=False,
+            action='store_true',
+            help=_("Print image size in a human-friendly format."),
+        )
         parser.add_argument(
             "image",
             metavar="<image>",
-            help="Image to display (name or ID)",
+            help=_("Image to display (name or ID)"),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        image_client = self.app.client_manager.image
+        image = utils.find_resource(
+            image_client.images,
+            parsed_args.image,
+        )
+        if parsed_args.human_readable:
+            image['size'] = utils.format_size(image['size'])
+
+        info = _format_image(image)
+        return zip(*sorted(six.iteritems(info)))
+
+
+class UnsetImage(command.Command):
+    _description = _("Unset image tags and properties")
+
+    def get_parser(self, prog_name):
+        parser = super(UnsetImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "image",
+            metavar="<image>",
+            help=_("Image to modify (name or ID)"),
+        )
+        parser.add_argument(
+            "--tag",
+            dest="tags",
+            metavar="<tag>",
+            default=[],
+            action='append',
+            help=_("Unset a tag on this image "
+                   "(repeat option to unset multiple tags)"),
+        )
+        parser.add_argument(
+            "--property",
+            dest="properties",
+            metavar="<property-key>",
+            default=[],
+            action='append',
+            help=_("Unset a property on this image "
+                   "(repeat option to unset multiple properties)"),
         )
         return parser
 
@@ -850,5 +1066,43 @@ class ShowImage(command.ShowOne):
             parsed_args.image,
         )
 
-        info = _format_image(image)
-        return zip(*sorted(six.iteritems(info)))
+        kwargs = {}
+        tagret = 0
+        propret = 0
+        if parsed_args.tags:
+            for k in parsed_args.tags:
+                try:
+                    image_client.image_tags.delete(image.id, k)
+                except Exception:
+                    LOG.error(_("tag unset failed, '%s' is a "
+                                "nonexistent tag "), k)
+                    tagret += 1
+
+        if parsed_args.properties:
+            for k in parsed_args.properties:
+                if k not in image:
+                    LOG.error(_("property unset failed, '%s' is a "
+                                "nonexistent property "), k)
+                    propret += 1
+            image_client.images.update(
+                image.id,
+                parsed_args.properties,
+                **kwargs)
+
+        tagtotal = len(parsed_args.tags)
+        proptotal = len(parsed_args.properties)
+        if (tagret > 0 and propret > 0):
+            msg = (_("Failed to unset %(tagret)s of %(tagtotal)s tags,"
+                   "Failed to unset %(propret)s of %(proptotal)s properties.")
+                   % {'tagret': tagret, 'tagtotal': tagtotal,
+                      'propret': propret, 'proptotal': proptotal})
+            raise exceptions.CommandError(msg)
+        elif tagret > 0:
+            msg = (_("Failed to unset %(tagret)s of %(tagtotal)s tags.")
+                   % {'tagret': tagret, 'tagtotal': tagtotal})
+            raise exceptions.CommandError(msg)
+        elif propret > 0:
+            msg = (_("Failed to unset %(propret)s of %(proptotal)s"
+                   " properties.")
+                   % {'propret': propret, 'proptotal': proptotal})
+            raise exceptions.CommandError(msg)
